using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Configuration;
using MediaPortal.GUI.Library;
using MediaPortal.Configuration;
using MediaPortal.Dialogs;


namespace Traffic
{
  public class TrafficGUI : GUIWindow, ISetupForm
  {
    #region Enums

    enum Mode
    {
      Map,
      Cam
    }

    enum InfoMode
    {
      Jams,
      Roadwork,
      Speedcams
    }

    #endregion


    private static string NationalURL = "http://www.anwb.nl/verkeer/verkeersinformatie_nederland.nl.html";
    private static string Region1URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_randstad.nl.html";
    private static string Region2URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_utrecht.nl.html";
    private static string Region3URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_n_nederland.nl.html";
    private static string Region4URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_nw_nederland.nl.html";
    private static string Region5URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_o_nederland.nl.html";
    private static string Region6URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_zon_nederland.nl.html";
    private static string Region7URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_zw_nederland.nl.html";
    private static string Region8URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_zoz_nederland.nl.html";
    private static string Region9URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_amsterdam.nl.html";
    private static string Region10URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_rotterdam.nl.html";
    private static string Region11URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_denhaag.nl.html";
    private static string Region12URL = "http://www.anwb.nl/verkeer/popup_regioverkeerssituatie_eindhoven.nl.html";


    private static string Region15URL = "http://www.dot.ca.gov/dist11/d11tmc/sdmap/speedtable.php?route=nb5";

    [SkinControlAttribute(2)]protected GUIButtonControl buttonRefresh = null;
    [SkinControlAttribute(10)]protected GUIButtonControl buttonRegion = null;
    [SkinControlAttribute(21)]protected GUIButtonControl buttonWebcam = null;
    [SkinControlAttribute(22)]protected GUISelectButtonControl buttonInformation = null;
    [SkinControlAttribute(7)]protected GUILabelControl labelStatus = null;
    [SkinControlAttribute(8)]protected GUITextScrollUpControl labelInfo = null;
    

    string BasisLayerLink = string.Empty;
    string JamLayerLink = string.Empty;
    string WeatherLayerLink = string.Empty;

    GUIImage BasisLayerImage = null;
    GUIImage JamLayerImage1 = null;
    GUIImage JamLayerImage2 = null;
    GUIImage SpeedCameraLayerImage = null;
    GUIImage RoadworkLayerImage = null;
    GUIImage WeatherLayerImage = null;
    GUIImage CamLayerImage = null;

    Region currentRegion = null;
    Cam currentCam = null;

    ArrayList Regions = new ArrayList();
    Mode currentMode = Mode.Map;
    InfoMode currentInfoMode = InfoMode.Jams;
    TrafficReader reader = new TrafficReader();

    private void Update()
    {
      currentRegion = reader.ReadTraffic(currentRegion, TrafficReader.Source.USA_SanDiego);

      BasisLayerImage = (GUIImage)GetControl(4);
      JamLayerImage1 = (GUIImage)GetControl(5);
      JamLayerImage2 = (GUIImage)GetControl(6);
      CamLayerImage = (GUIImage)GetControl(11);
      RoadworkLayerImage = (GUIImage)GetControl(12);
      SpeedCameraLayerImage = (GUIImage)GetControl(13);
      WeatherLayerImage = (GUIImage)GetControl(14);

      if (currentRegion.Cams.Count == 0)
      {
        buttonWebcam.Disabled = true;
        
      }
      else if (currentRegion.Cams.Count > 0)
      {
        buttonWebcam.Disabled = false;

      }

      GUIControl.SetControlLabel(GetID, 7, "Status " + currentRegion.Status);
      string st = string.Empty;

      if (currentInfoMode == InfoMode.Jams)
      {
        if (currentRegion.Jams.Count > 0)
        {
          for (int i = 0; i < currentRegion.Jams.Count; i++)
          {
            st = st + " * " +  ((Jam)(currentRegion.Jams[i])).Road + " " + ((Jam)(currentRegion.Jams[i])).Description + " " + ((Jam)(currentRegion.Jams[i])).Length + "\r\n";
          }
        }
        else
        {
          if (currentRegion.Reports != string.Empty)
          {
            st = currentRegion.Reports + " reports with total length of " + currentRegion.TotalLength;
          }
          else
          {
            st = "No traffic jams";
          }
        }
      }
      else if(currentInfoMode == InfoMode.Roadwork)
      {
        if (currentRegion.Roadworks.Count > 0)
        {
          for (int i = 0; i < currentRegion.Roadworks.Count; i++)
          {
            st = st + " * " +  ((Roadwork)(currentRegion.Roadworks[i])).Road + " " + ((Roadwork)(currentRegion.Roadworks[i])).Description + "\r\n";
          }
        }
        else
        {
          st = "No roadwork";
        }
      }
      else if (currentInfoMode == InfoMode.Speedcams)
      {
        if (currentRegion.Speedcams.Count > 0)
        {
          for (int i = 0; i < currentRegion.Speedcams.Count; i++)
          {
            st = st + " * " + ((SpeedCam)(currentRegion.Speedcams[i])).Road + " " + ((SpeedCam)(currentRegion.Speedcams[i])).Description + "\r\n";
          }
        }
        else
        {
          st = "No speedcams";
        }
      }

      GUIControl.SetControlLabel(GetID, 8, st);

      if (currentMode == Mode.Map)
      {

        if (currentRegion.BasisLayerLink != string.Empty)
        {
          BasisLayerImage.SetFileName(currentRegion.BasisLayerLink);
          
          BasisLayerImage.Height = 470;
          BasisLayerImage.Width = 500;
          BasisLayerImage.Visible = true;
        }

        if (currentRegion.JamLayerLink1 != string.Empty)
        {
          JamLayerImage1.SetFileName(currentRegion.JamLayerLink1);
          JamLayerImage1.Visible = true;
        }

        if (currentRegion.WeatherLayerLink != string.Empty)
        {
          WeatherLayerImage.SetFileName(currentRegion.WeatherLayerLink);
          WeatherLayerImage.Visible = true;
        }

        if (currentRegion.JamLayerLink2 != string.Empty)
        {
          JamLayerImage2.SetFileName(currentRegion.JamLayerLink2);
          JamLayerImage2.Visible = true;
        }

        if (currentRegion.CamLayerLink != string.Empty)
        {
          CamLayerImage.SetFileName(currentRegion.CamLayerLink);
          CamLayerImage.Visible = true;
        }

        if (currentRegion.SpeedCameraLayerLink != string.Empty)
        {
          SpeedCameraLayerImage.SetFileName(currentRegion.SpeedCameraLayerLink);
          SpeedCameraLayerImage.Visible = true;
        }

        if (currentRegion.RoadworkLayerLink != string.Empty)
        {
          RoadworkLayerImage.SetFileName(currentRegion.RoadworkLayerLink);
          RoadworkLayerImage.Visible = true;
        }
      }
      else if (currentMode == Mode.Cam)
      {
        BasisLayerImage.SetFileName(currentCam.URL);

        JamLayerImage1.Visible = false;
        JamLayerImage2.Visible = false;

        CamLayerImage.Visible = false;
        RoadworkLayerImage.Visible = false;
        SpeedCameraLayerImage.Visible = false;
        WeatherLayerImage.Visible = false;

        GUIControl.SetControlLabel(GetID, 7, "Status " + currentCam.Location);
      }

    }

    protected override void OnPageLoad()
    {/*
      Region r1 = new Region(NationalURL);
      r1.Name = "Nederland";

      Region r2 = new Region(Region1URL);
      r2.Name = "Randstad";
      r2.Cams.Add(new Cam("Holendrecht", "A2/A9 Knp. Holendrecht"));
      r2.Cams.Add(new Cam("NieuweMeer", "A4/A10 Knp. Nieuwe Meer"));
      r2.Cams.Add(new Cam("Badhoevedorp", "A4/A9 Knp. Badhoevedorp"));
      r2.Cams.Add(new Cam("Schiphol", "A4 Schiphol"));
      r2.Cams.Add(new Cam("Burgerveen", "A4/A44 Knp. Burgerveen"));
      r2.Cams.Add(new Cam("Kleinpolderplein1", "A20 Knp. Kleinpolderplein"));
      r2.Cams.Add(new Cam("Kleinpolderplein2", "A13 Knp. Kleinpolderplein"));
      r2.Cams.Add(new Cam("Kleinpolderplein3", "A13 Knp. Kleinpolderplein"));
      r2.Cams.Add(new Cam("Kleinpolderplein4", "A20 Knp. Kleinpolderplein"));
      r2.Cams.Add(new Cam("Hoevelaken", "A1/A28 Knp. Hoevelaken"));
      r2.Cams.Add(new Cam("Ouderijn", "A2/A12 Knp. Oudenrijn"));
      r2.Cams.Add(new Cam("Lunetten", "A12/A27 Knp. Lunetten"));
      r2.Cams.Add(new Cam("Everdingen", "A2/A27 Knp. Everdingen"));
      r2.Cams.Add(new Cam("Rijnsweerd", "A27/A28 Knp. Rijnsweerd"));

      Region r3 = new Region(Region2URL);
      r3.Name = "Utrecht";
      r3.Cams.Add(new Cam("Ouderijn", "A2/A12 Knp. Oudenrijn"));
      r3.Cams.Add(new Cam("Lunetten", "A12/A27 Knp. Lunetten"));
      r3.Cams.Add(new Cam("Everdingen", "A2/A27 Knp. Everdingen"));
      r3.Cams.Add(new Cam("Rijnsweerd", "A27/A28 Knp. Rijnsweerd"));

      Region r4 = new Region(Region3URL);
      r4.Name = "Noord-Nederland";

      Region r5 = new Region(Region4URL);
      r5.Name = "Noordwest-Nederland";
      r5.Cams.Add(new Cam("Holendrecht", "A2/A9 Knp. Holendrecht"));
      r5.Cams.Add(new Cam("NieuweMeer", "A4/A10 Knp. Nieuwe Meer"));
      r5.Cams.Add(new Cam("Badhoevedorp", "A4/A9 Knp. Badhoevedorp"));
      r5.Cams.Add(new Cam("Schiphol", "A4 Schiphol"));

      Region r6 = new Region(Region5URL);
      r6.Name = "Oost-Nederland";
      r6.Cams.Add(new Cam("Hoevelaken", "A1/A28 Knp. Hoevelaken"));

      Region r7 = new Region(Region6URL);
      r7.Name = "Zuidoost-Nederland";
      r7.Cams.Add(new Cam("Ewijk", "A50/A73 Knp. Ewijk"));
      r7.Cams.Add(new Cam("Ouderijn", "A2/A12 Knp. Oudenrijn"));
      r7.Cams.Add(new Cam("Lunetten", "A12/A27 Knp. Lunetten"));
      r7.Cams.Add(new Cam("Everdingen", "A2/A27 Knp. Everdingen"));

      Region r8 = new Region(Region7URL);
      r8.Name = "Zuidwest-Nederland";
      r8.Cams.Add(new Cam("Kleinpolderplein1", "A20 Knp. Kleinpolderplein"));
      r8.Cams.Add(new Cam("Kleinpolderplein2", "A13 Knp. Kleinpolderplein"));
      r8.Cams.Add(new Cam("Kleinpolderplein3", "A13 Knp. Kleinpolderplein"));
      r8.Cams.Add(new Cam("Kleinpolderplein4", "A20 Knp. Kleinpolderplein"));

      Region r9 = new Region(Region8URL);
      r9.Name = "Zuid-Nederland, Belgie";

      Region r10 = new Region(Region9URL);
      r10.Name = "Amsterdam";
      r10.Cams.Add(new Cam("Holendrecht", "A2/A9 Knp. Holendrecht"));
      r10.Cams.Add(new Cam("NieuweMeer", "A4/A10 Knp. Nieuwe Meer"));
      r10.Cams.Add(new Cam("Badhoevedorp", "A4/A9 Knp. Badhoevedorp"));
      r10.Cams.Add(new Cam("Schiphol", "A4 Schiphol"));

      Region r11 = new Region(Region10URL);
      r11.Name = "Rotterdam";
      r11.Cams.Add(new Cam("Kleinpolderplein1", "A20 Knp. Kleinpolderplein"));
      r11.Cams.Add(new Cam("Kleinpolderplein2", "A13 Knp. Kleinpolderplein"));
      r11.Cams.Add(new Cam("Kleinpolderplein3", "A13 Knp. Kleinpolderplein"));
      r11.Cams.Add(new Cam("Kleinpolderplein4", "A20 Knp. Kleinpolderplein"));

      Region r12 = new Region(Region11URL);
      r12.Name = "Den Haag";
      r12.Cams.Add(new Cam("Kleinpolderplein1", "A20 Knp. Kleinpolderplein"));
      r12.Cams.Add(new Cam("Kleinpolderplein2", "A13 Knp. Kleinpolderplein"));
      r12.Cams.Add(new Cam("Kleinpolderplein3", "A13 Knp. Kleinpolderplein"));
      r12.Cams.Add(new Cam("Kleinpolderplein4", "A20 Knp. Kleinpolderplein"));

      Region r13 = new Region(Region12URL);
      r13.Name = "Eindhoven";


      Regions.Add(r1);
      Regions.Add(r2);
      Regions.Add(r4);
      Regions.Add(r5);
      Regions.Add(r6);
      Regions.Add(r7);
      Regions.Add(r8);
      Regions.Add(r9);
      Regions.Add(r10);
      Regions.Add(r11);
      Regions.Add(r12);
      Regions.Add(r3);
      Regions.Add(r13);
      
      */

      Region r = new Region(Region15URL);
      r.Name = "San Diego";
      r.BasisLayerLink = "http://www.dot.ca.gov/dist11/d11tmc/sdmap/realtdata/realtmap.jpg";
      r.Cams.Add(new Cam("http://www.dot.ca.gov/dist11/d11tmc/sdmap/cameras/cam2.jpg", "I-805 Towards Route 5/805 Merge"));
      r.Cams.Add(new Cam("http://www.dot.ca.gov/dist11/d11tmc/sdmap/cameras/cam3.jpg", "II-5 Towards Route 5/805 Merge"));
      r.Cams.Add(new Cam("http://www.dot.ca.gov/dist11/d11tmc/sdmap/cameras/cam4.jpg", "I-5 Towards Route 5/805 Interchange"));
      r.Cams.Add(new Cam("http://www.dot.ca.gov/dist11/d11tmc/sdmap/cameras/cam5.jpg", "I-5 Towards Del Mar Heights Rd."));

      Regions.Add(r);

      currentRegion = (Region)Regions[0];
      Update();
    }

    public override bool Init()
    {
      return Load(GUIGraphicsContext.Skin + @"\Traffic.xml");
    }
    
    protected override void OnClicked(int controlId, GUIControl control, MediaPortal.GUI.Library.Action.ActionType actionType)
    {
      if (control == buttonInformation)
      {
          GUIMessage msg = new GUIMessage(GUIMessage.MessageType.GUI_MSG_ITEM_SELECTED, GetID, 0, controlId, 0, 0, null);
          OnMessage(msg);

          if ((msg.Label).Equals("Traffic jams"))
          {
            currentInfoMode = InfoMode.Jams;

          }
          if ((msg.Label).Equals("Roadwork"))
          {
            currentInfoMode = InfoMode.Roadwork;
          }
          if ((msg.Label).Equals("Speedcams"))
          {
            currentInfoMode = InfoMode.Speedcams;
          }
          Update();
          GUIControl.FocusControl(GetID, buttonInformation.GetID);

      }
      
      if (control == buttonRefresh)
      {
        Update();
      }

      if (control == buttonWebcam)
      {
        GUIDialogMenu dialogOk = (GUIDialogMenu)GUIWindowManager.GetWindow((int)GUIWindow.Window.WINDOW_DIALOG_MENU);
        if (dialogOk != null)
        {
          dialogOk.Reset();
          dialogOk.SetHeading(496); // menu

          for (int i = 0; i < currentRegion.Cams.Count; i++)
          {
            dialogOk.Add(new GUIListItem( (((Cam)currentRegion.Cams[i]).Location)));
          }

          dialogOk.DoModal(GetID);
          if (dialogOk.SelectedLabel == -1)
            return;

          currentCam = ((Cam)(currentRegion.Cams[dialogOk.SelectedLabel]));

          currentMode = Mode.Cam;

          Update();

        }
      }

      if (control == buttonRegion)
      {
          GUIDialogMenu dialogOk = (GUIDialogMenu)GUIWindowManager.GetWindow((int)GUIWindow.Window.WINDOW_DIALOG_MENU);
          if (dialogOk != null)
          {
            dialogOk.Reset();
            dialogOk.SetHeading(496); // menu

            for (int i = 0; i < Regions.Count; i++)
            {
              dialogOk.Add(new GUIListItem((((Region)Regions[i]).Name)));
            }

            dialogOk.DoModal(GetID);
            if (dialogOk.SelectedLabel == -1)
              return;

            currentRegion = ((Region)(Regions[dialogOk.SelectedLabel]));

            currentMode = Mode.Map;

            Update();

          }
      }

      base.OnClicked(controlId, control, actionType);

    }

    public TrafficGUI()
    {
      GetID = 9662;
    }

    #region ISetupForm Members

    public string PluginName()
    {
      return "Traffic";
    }

    // Returns the description of the plugin is shown in the plugin menu
    public string Description()
    {
      return "Shows ANWB Traffic Information";
    }

    // Returns the author of the plugin which is shown in the plugin menu
    public string Author()
    {
      return "achappey";
    }

    // show the setup dialog
    public void ShowPlugin()
    {
      MessageBox.Show("Just an example");
    }

    // Indicates whether plugin can be enabled/disabled
    public bool CanEnable()
    {
      return true;
    }

    // Enter the id number here again
    public int GetWindowId()
    {
      return 9662;
    }
    // Indicates if plugin is enabled by default;
    public bool DefaultEnabled()
    {
      return true;
    }

    // indicates if a plugin has it's own setup screen
    public bool HasSetup()
    {
      return false;
    }

    /// <summary>
    /// If the plugin should have it's own button on the main menu of MediaPortal then it
    /// should return true to this method, otherwise if it should not be on home
    /// it should return false
    /// </summary>
    /// <param name="strButtonText">text the button should have</param>
    /// <param name="strButtonImage">image for the button, or empty for default</param>
    /// <param name="strButtonImageFocus">image for the button, or empty for default</param>
    /// <param name="strPictureImage">subpicture for the button or empty for none</param>
    /// <returns>true : plugin needs it's own button on home
    /// false : plugin does not need it's own button on home</returns>

    public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
    {
      strButtonText = PluginName();
      strButtonImage = String.Empty;
      strButtonImageFocus = String.Empty;
      strPictureImage = String.Empty;
      return true;
    }

    #endregion
  }
}
